/* SingleSignOn support for HP OpenView ServiceDesk 4.5
 * Copyright (C) 2008  "Radovan Skolnik" <radovan at skolnik dot info>
 *
 */

package info.skolnik.radovan.hpovsd45.sso;

import com.hp.itsm.ssp.beans.SdClientBean;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;

/**
 * This servlet Filter uses data from jcifs.http.NtlmHttpFilter
 * that should be previous in the Filter chain (see web.xml)
 * to "inject" (if not already there) pre-authenticated SdClientBean into session.
 * The SdClientBean is retrieved from SdClientBeanHelper provided as part of the SSO package
 */

public class HttpFilter implements Filter {

    protected static String SINGLE_SIGN_ON = "sd-client-bean";
    protected static String serviceDeskServer = null;
    protected static String webApiSecretKey = null;
    protected static boolean debug = false;

    /**
     * We just initialize som global stuff here tht we'll use in individual requests
     */
    public void init( FilterConfig filterConfig ) throws ServletException {
        serviceDeskServer = filterConfig.getServletContext().getInitParameter("sd_application_server"); // this is how it's named in web.xml for sd-sp45 application
        webApiSecretKey = filterConfig.getInitParameter("webApiSecretKey"); // this is our filter's parameter
        debug = (Boolean.valueOf(filterConfig.getInitParameter("debug"))).booleanValue(); // this is our filter's parameter
    }

    public void destroy() {
        // nothing in here
    }

    /**
     * This method checks if we have existig bean in the session
     * If not it attempts to create one useing SdClientBeanHelper
     * and information that should be provided by jcifs.http.NtlmHttpFilter
     */
    public void doFilter( ServletRequest request, ServletResponse response, FilterChain chain ) throws IOException, ServletException {
        try {
            // check if there is a bean in the session

            // first we need to cast to a HTTP servlet request (wrapper) so that we can retrieve various info from it
            HttpServletRequestWrapper httpRequest = (HttpServletRequestWrapper) request;
            SdClientBean sessionBean =  (SdClientBean) httpRequest.getSession().getAttribute(SINGLE_SIGN_ON);
            if (sessionBean != null) {
                // we already have a bean registered in the session
            } else {
                // we do not have bean registered so let's try to retrieve one and inject it

                // if properly configured and authenticated, this should contain remote user identification in the form of DOMAIN\LOGIN
                String remoteUser = httpRequest.getRemoteUser();

                if ((remoteUser != null) && (! remoteUser.trim().equals(""))) {
                    // parse domain and login name
                    String domain = null;
                    String login = null;
                    try {
                        StringTokenizer st = new StringTokenizer(remoteUser, "\\");
                        domain = st.nextToken();
                        login = st.nextToken();
                    } catch (NoSuchElementException nsee) {
                        // maybe do something and not just ignore?
                    }

                    // we fill in variables that will be passed to AFQ on server side
                    Hashtable variables = new Hashtable();
                    variables.put("$NT_DOMAIN$", domain);
                    variables.put("$NT_NAME$", login);

                    // this uses WEB-API AFQ but instead provides pre-authenitcated SdClientBean (which is in fact only wrapper around ApiSDSession)
                    sessionBean = SdClientBeanHelper.getSdClientBean(serviceDeskServer, webApiSecretKey, variables, debug);
                    if (sessionBean != null) {
                        // we have got one, let's inject it into session
                        httpRequest.getSession().setAttribute(SINGLE_SIGN_ON, sessionBean);
                    }
                } else {
                    throw new SSOException("Failed to retrieve remote user identification");
                }
            }
        } catch (Exception ex) {
            throw new ServletException(ex);
        }

        // forward the request further (in case you add some additional filter)
        chain.doFilter( request, response );
    }

    // these 2 methods are copied from jcifs.http.NtlmHttpFilter - not needed if deployed in Tomcat
    // Added by cgross to work with weblogic 6.1.
    public void setFilterConfig( FilterConfig f ) {
        try {
            init( f );
        } catch( Exception e ) {
            e.printStackTrace();
        }
    }
    public FilterConfig getFilterConfig() {
        return null;
    }
}

